/*
 * Project name:
     WiFI3 click board example;
 * Copyright:
     (c) MikroElektronika, 2015.
  * Revision History:
     20150126:
       - initial release (FJ);
 * Description:
     This is a simple project which demonstrates the use of WiFi3 click board.
     After establishing the connection, user should send messages via UDP terminal
     The WiFI3 click board receives them and displays them at the TFT.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/reference_manual/CD00171190.pdf
     Dev. Board:      EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place WiFi3 click board in the mikroBUS socket 1 on the EasyMX PRO v7 board.
     - Put EasyTFT in its socket, turn on backlight using the SW11.3
     - Configure SSId, password and port to match your network settings.
     - Use UDP terminal tool to connect to the WiFi module and send data to it.
 */

#include "AT_Commands.h"

//Constants

const char SSID[] = "MikroE Private";
const char password[] = "mikroELEKTRONIKA";

const char port[] = "8080";
const char msg_hdr[] = "+IPD";

#define CR           0x0D
#define LF           0x0A
#define NEW_CONFIGURATION

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

sfr sbit WiFi3_PWD at GPIOA_ODR.B4;
sfr sbit WiFi2_GP15 at GPIOC_ODR.B2;


char rx_buff[64];
char data_ready;
char data_len, tmp;

void UART3_Interrupt() iv IVT_INT_USART3 ics ICS_AUTO {
   tmp = UART3_Read();
   switch(tmp) {
     case LF:
        if(data_len > 1){
          data_ready = 1;
        }
        break;
     case CR:
        rx_buff[data_len] = 0; //but not necessary
        break;
     default:
        rx_buff[data_len] = tmp;
        data_len++;
   }
   
}

void UART_Write_AT(char *CMD) {
   UART3_Write_Text(CMD);
   UART3_Write(0x0D);
   UART3_Write(0x0A);
}

void reset_buff() {
  memset(rx_buff,0,data_len);
  data_ready = 0;
  data_len=0;
}

char response_success() {
    char result;
    while(!data_ready);
    if((strstr(rx_buff,"OK")) || (strstr(rx_buff,"no change")) || (strstr(rx_buff,"ready"))) {
       result = 1;
    }else result = 0;
    reset_buff();
    return result;
}



void Get_IP() {
   char response_cnt, conn_flg;

     TFT_Fill_Screen(CL_BLUE);
     TFT_Write_Text("Getting IP Address",10,10);
     TFT_Write_Text("Please wait...",10,40);
     UART_Write_AT("AT+CIFSR");
     while(!data_ready);
     reset_buff();
     while(!data_ready);
     TFT_Write_Text(rx_buff,10,70);  //this is the IP address
     reset_buff();
}


void Wifi3_Click_Config() {
   char ap_cnt = 0;
   memset(rx_buff,0,64);
   TFT_Write_Text("Resetting module",10,10);
   TFT_Write_Text("Please wait...",10,40);
   UART_Write_AT("AT+RST");                                                     //Reset module
   while(!response_success());
   delay_ms(1000);
   //
   TFT_Fill_Screen(CL_BLUE);
   TFT_Write_Text("Setting STA mode",10,10);
   TFT_Write_Text("Please wait...",10,40);
   UART_Write_AT("AT+CWMODE=1");                                                //change the working mode to 1
   while(!response_success());                                                  //wait to respond success
   delay_ms(1000);
   //
   TFT_Fill_Screen(CL_BLUE);
   TFT_Write_Text("Setting connection mode",10,10);
   TFT_Write_Text("Please wait...",10,40);
   UART_Write_AT("AT+CIPMUX=1");                                                //change the connection mode to 1
   while(!response_success());                                                  //wait to respond succes
   delay_ms(1000);
   //
   while(1) {                                                                   //Join to AP
     TFT_Fill_Screen(CL_BLUE);
     TFT_Write_Text("Connecting to AP",10,10);
     TFT_Write_Text("Please wait...",10,40);
     UART3_Write_Text("AT+CWJAP=");
     UART3_Write('"');
     UART3_Write_Text(SSID);
     UART3_Write('"');
     UART3_Write(',');
     UART3_Write('"');
     UART3_Write_Text(password);
     UART3_Write('"');
     UART_Write(CR);
     UART_Write(LF);
     if(response_success()){
       TFT_Write_Text("Success",10,70);
       break;
     }else{
       TFT_Write_Text("Try again...",10,70);
       delay_ms(2000);
     }
   }
   delay_ms(1000);
   //test
   TFT_Fill_Screen(CL_BLUE);
   TFT_Write_Text("Check connection",10,10);
   TFT_Write_Text("Please wait...",10,40);
   UART3_Write_Text("AT+CWJAP=?");
   while(!response_success());
   TFT_Write_Text("WiFi3 Click Connected",10,70);
   delay_ms(1000);
   //end test
   //
   TFT_Fill_Screen(CL_BLUE);
   TFT_Write_Text("Configure TCP server",10,10);
   TFT_Write_Text("Please wait...",10,40);
   UART3_Write_Text("AT+CIPSERVER=1");                                          //Configure TCP server
   UART3_Write(',');
   UART3_Write_Text(port);
   UART3_Write(CR);
   UART3_Write(LF);
   while(!response_success());
   delay_ms(1000);
   //
   Get_IP();                                                                     //Get IP address or check connection
}


void Display_Init(){
  TFT_Init_ILI9341_8bit(320, 240);
  TFT_BLED = 1;
  TFT_Set_Pen(CL_BLUE, 1);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Set_Brush(1,CL_BLUE,0,0,0,0);
  TFT_Fill_Screen(CL_BLUE);
}

void Display_Message() {
  TFT_Fill_Screen(CL_BLUE);
  TFT_Write_Text(rx_buff,10,10);
  reset_buff();
}

void main() {
  GPIO_Digital_Output (&GPIOA_BASE, _GPIO_PINMASK_4);
  GPIO_Digital_Output (&GPIOC_BASE, _GPIO_PINMASK_2);
  Display_Init();
  UART3_Init_Advanced(9600, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART3_PD89);
  delay_ms(200);
  WiFi3_PWD = 1;
  delay_ms(500);
  data_ready = 0;
  data_len = 0;
  RXNEIE_USART3_CR1_bit = 1;
  NVIC_IntEnable(IVT_INT_USART3);
  #ifdef NEW_CONFIGURATION
  WiFi3_Click_Config();
  #else
  Get_IP();
  #endif
  while(1) {
    if(data_ready) {
      if(strstr(rx_buff,msg_hdr)) {
         Display_Message();
      }
    }
  }
}